import _pick from 'lodash/pick';
import mgl from 'mapbox-gl';
export { default as mgl } from 'mapbox-gl';
import Vue from 'vue';
import _isEqual from 'lodash/isEqual';
import _omit from 'lodash/omit';
import _get from 'lodash/get';
import _without from 'lodash/without';
import _mapValues from 'lodash/mapValues';
import _upperFirst from 'lodash/upperFirst';
import _mapKeys from 'lodash/mapKeys';
import _flow from 'lodash/flow';

const Observer = new Vue().$data.__ob__.constructor;
function preventObserve(val) {
  if (val) {
    // Set dummy observer on value
    Object.defineProperty(val, '__ob__', {
      value: new Observer({}),
      enumerable: false,
      configurable: true
    });
  }

  return val;
}

var OptionsMixin = {
  props: {
    token: {
      type: String,
      required: true
    },
    container: {
      type: [String, HTMLElement]
    },
    minZoom: {
      type: Number,
      default: 0
    },
    maxZoom: {
      type: Number,
      default: 22
    },
    minPitch: {
      type: Number,
      default: 0
    },
    maxPitch: {
      type: Number,
      default: 60
    },
    // [Vue warn]: "style" is a reserved attribute and cannot be used as component prop.
    mapStyle: {
      type: [String, Object],
      default: () => 'https://api.cedarmaps.com/v1/styles/cedarmaps.light.json'
    },
    hash: {
      type: Boolean,
      default: false
    },
    interactive: {
      type: Boolean,
      default: true
    },
    bearingSnap: {
      type: Number,
      default: 7
    },
    pitchWithRotate: {
      type: Boolean,
      default: true
    },
    clickTolerance: {
      type: Number,
      default: 3
    },
    attributionControl: {
      type: Boolean,
      default: true
    },
    customAttribution: {
      type: [String, Array]
    },
    logoPosition: {
      type: String,
      default: 'bottom-left'
    },
    failIfMajorPerformanceCaveat: {
      type: Boolean,
      default: false
    },
    preserveDrawingBuffer: {
      type: Boolean,
      default: false
    },
    antialias: {
      type: Boolean,
      default: false
    },
    refreshExpiredTiles: {
      type: Boolean,
      default: true
    },
    maxBounds: {
      type: Array
    },

    /**
     * interactive handler
     */
    scrollZoom: {
      type: [Boolean, Object],
      default: true
    },
    boxZoom: {
      type: Boolean,
      default: true
    },
    dragRotate: {
      type: Boolean,
      default: true
    },
    dragPan: {
      type: Boolean,
      default: true
    },
    keyboard: {
      type: Boolean,
      default: true
    },
    doubleClickZoom: {
      type: Boolean,
      default: true
    },
    touchZoomRotate: {
      type: [Boolean, Object],
      default: true
    },
    trackResize: {
      type: Boolean,
      default: true
    },

    /**
     * view
     */
    center: {
      type: [Array, Object],
      default: () => [0, 0]
    },
    zoom: {
      type: Number,
      default: 0
    },
    bearing: {
      type: Number,
      default: 0
    },
    pitch: {
      type: Number,
      default: 0
    },
    bounds: {
      type: [Array, Object]
    },
    fitBoundsOptions: {
      type: Object
    },

    /**
     * other
     */
    renderWorldCopies: {
      type: Boolean,
      default: true
    },
    maxTileCacheSize: {
      type: Number,
      default: null
    },
    localIdeographFontFamily: {
      type: String,
      default: null
    },
    transformRequest: {
      type: Function,
      default: null
    },
    collectResourceTiming: {
      type: Boolean,
      default: false
    },
    fadeDuration: {
      type: Number,
      default: 300
    },
    crossSourceCollisions: {
      type: Boolean,
      default: true
    },
    accessToken: {
      type: String
    },
    locale: {
      type: String
    }
  },
  watch: {
    zoom(val, old) {
      if (!this.ready) return;
      if (_isEqual(val, old)) return;
      this.map.setZoom(val);
    },

    center(val, old) {
      if (!this.ready) return;
      if (_isEqual(val, old)) return;
      this.map.setCenter(val);
    },

    bearing(val, old) {
      if (!this.ready) return;
      if (_isEqual(val, old)) return;
      this.map.setBearing(val);
    },

    pitch(val, old) {
      if (!this.ready) return;
      if (_isEqual(val, old)) return;
      this.map.setPitch(val);
    },

    minZoom(zoom) {
      if (!this.ready) return;
      this.map.setMinZoom(zoom);
    },

    maxZoom(zoom) {
      if (!this.ready) return;
      this.map.setMaxZoom(zoom);
    },

    maxBounds(bounds) {
      if (!this.ready) return;
      this.map.setMaxBounds(bounds);
    }

  }
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var script = {
  mixins: [OptionsMixin],

  provide() {
    return {
      getMapComponent: this.getMapComponent
    };
  },

  data() {
    return {
      ready: false,
      map: null
    };
  },

  watch: {
    mapStyle(val) {
      // set to map
      if (!this.ready) return; // https://github.com/mapbox/mapbox-gl-js/blob/v0.42.2/src/ui/map.js#L919
      // https://github.com/mapbox/mapbox-gl-js/blob/v0.42.2/src/style/style.js#L230
      //
      // new version
      // style.load will buble to map

      this.map.once('style.load', () => {
        this.$emit('style-load');
      });
      this.map.setStyle(val);
    }

  },

  mounted() {
    this.createMap();
  },

  destroyed() {
    if (this.map) this.map.remove();
  },

  methods: {
    getMap() {
      return this.map;
    },

    getMapComponent() {
      return this;
    },

    createMap() {
      const optionsKeys = Object.keys(OptionsMixin.props);

      const options = _pick(this.$props, optionsKeys); // container


      if (!options.container) options.container = this.$refs.container; // style

      options.style = `${options.mapStyle}?access_token=${options.token}`;
      delete options.mapStyle;
      const map = new mgl.Map(options);
      preventObserve(map);
      this.map = map;
      map.on('load', () => {
        this.setupUpdateProps();

        if (mgl.getRTLTextPluginStatus() === 'unavailable') {
          mgl.setRTLTextPlugin('https://api.mapbox.com/mapbox-gl-js/plugins/mapbox-gl-rtl-text/v0.2.0/mapbox-gl-rtl-text.js');
        }

        this.ready = true;
        this.$emit('ready');
        this.$emit('load', {
          map,
          component: this
        });
      });
      map.on('error', err => {
        this.$emit('error', err);

        if (!this.$listeners.error) {
          console.error(err.error.message || err.error.stack);
        }
      });
    },

    setupUpdateProps() {
      this.map.on('moveend', event => {
        this.$emit('update:center', this.map.getCenter());
      });
      this.map.on('zoomend', event => {
        this.$emit('update:zoom', this.map.getZoom());
      });
      this.map.on('rotate', event => {
        this.$emit('update:bearing', this.map.getBearing());
      });
      this.map.on('pitch', event => {
        this.$emit('update:pitch', this.map.getPitch());
      });
    }

  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

const isOldIE = typeof navigator !== 'undefined' &&
    /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
function createInjector(context) {
    return (id, style) => addStyle(id, style);
}
let HEAD;
const styles = {};
function addStyle(id, css) {
    const group = isOldIE ? css.media || 'default' : id;
    const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
    if (!style.ids.has(id)) {
        style.ids.add(id);
        let code = css.source;
        if (css.map) {
            // https://developer.chrome.com/devtools/docs/javascript-debugging
            // this makes source maps inside style tags work properly in Chrome
            code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
            // http://stackoverflow.com/a/26603875
            code +=
                '\n/*# sourceMappingURL=data:application/json;base64,' +
                    btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
                    ' */';
        }
        if (!style.element) {
            style.element = document.createElement('style');
            style.element.type = 'text/css';
            if (css.media)
                style.element.setAttribute('media', css.media);
            if (HEAD === undefined) {
                HEAD = document.head || document.getElementsByTagName('head')[0];
            }
            HEAD.appendChild(style.element);
        }
        if ('styleSheet' in style.element) {
            style.styles.push(code);
            style.element.styleSheet.cssText = style.styles
                .filter(Boolean)
                .join('\n');
        }
        else {
            const index = style.ids.size - 1;
            const textNode = document.createTextNode(code);
            const nodes = style.element.childNodes;
            if (nodes[index])
                style.element.removeChild(nodes[index]);
            if (nodes.length)
                style.element.insertBefore(textNode, nodes[index]);
            else
                style.element.appendChild(textNode);
        }
    }
}

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c("div", { ref: "container", staticClass: "mgl-map-wrapper" }, [
    _vm.ready
      ? _c("div", { staticClass: "more" }, [_vm._t("default")], 2)
      : _vm._e()
  ])
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = function (inject) {
    if (!inject) return
    inject("data-v-fca1289e_0", { source: "\n.mapboxgl-canvas {\n  right: 0;\n}\n", map: {"version":3,"sources":["/home/pamenari/Desktop/vue-mgl-master/src/component/Cedarmaps/index.vue"],"names":[],"mappings":";AAyHA;EACA,QAAA;AACA","file":"index.vue","sourcesContent":["<template>\n  <div class=\"mgl-map-wrapper\" ref=\"container\">\n    <div class=\"more\" v-if=\"ready\"><slot></slot></div>\n  </div>\n</template>\n\n<script>\nimport mgl from 'mapbox-gl'\nimport Vue from 'vue'\nimport { isEqual, pick } from 'lodash'\nimport preventObserve from '../../util/preventObserve.js'\n\n// part\nimport OptionsMixin from './options.mixin.js'\nimport MoveMixin from './move.mixin.js'\n\nexport default {\n  mixins: [OptionsMixin],\n  provide() {\n    return {\n      getMapComponent: this.getMapComponent,\n    }\n  },\n\n  data() {\n    return {\n      ready: false,\n      map: null,\n    }\n  },\n\n  watch: {\n    mapStyle(val) {\n      // set to map\n      if (!this.ready) return\n\n      // https://github.com/mapbox/mapbox-gl-js/blob/v0.42.2/src/ui/map.js#L919\n      // https://github.com/mapbox/mapbox-gl-js/blob/v0.42.2/src/style/style.js#L230\n      //\n      // new version\n      // style.load will buble to map\n      this.map.once('style.load', () => {\n        this.$emit('style-load')\n      })\n\n      this.map.setStyle(val)\n    },\n  },\n\n  mounted() {\n    this.createMap()\n  },\n\n  destroyed() {\n    if (this.map) this.map.remove()\n  },\n\n  methods: {\n    getMap() {\n      return this.map\n    },\n    getMapComponent() {\n      return this\n    },\n\n    createMap() {\n      const optionsKeys = Object.keys(OptionsMixin.props)\n      const options = pick(this.$props, optionsKeys)\n\n      // container\n      if (!options.container) options.container = this.$refs.container\n\n      // style\n      options.style = `${options.mapStyle}?access_token=${options.token}`\n      delete options.mapStyle\n\n      const map = new mgl.Map(options)\n      preventObserve(map)\n      this.map = map\n\n      map.on('load', () => {\n        this.setupUpdateProps()\n\n        if (mgl.getRTLTextPluginStatus() === 'unavailable') {\n          mgl.setRTLTextPlugin(\n            'https://api.mapbox.com/mapbox-gl-js/plugins/mapbox-gl-rtl-text/v0.2.0/mapbox-gl-rtl-text.js'\n          )\n        }\n\n        this.ready = true\n        this.$emit('ready')\n        this.$emit('load', { map, component: this })\n      })\n\n      map.on('error', err => {\n        this.$emit('error', err)\n        if (!this.$listeners.error) {\n          console.error(err.error.message || err.error.stack)\n        }\n      })\n    },\n\n    setupUpdateProps() {\n      this.map.on('moveend', event => {\n        this.$emit('update:center', this.map.getCenter())\n      })\n      this.map.on('zoomend', event => {\n        this.$emit('update:zoom', this.map.getZoom())\n      })\n      this.map.on('rotate', event => {\n        this.$emit('update:bearing', this.map.getBearing())\n      })\n      this.map.on('pitch', event => {\n        this.$emit('update:pitch', this.map.getPitch())\n      })\n    },\n  },\n}\n</script>\n\n<style>\n.mapboxgl-canvas {\n  right: 0;\n}\n</style>\n"]}, media: undefined });

  };
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    createInjector,
    undefined,
    undefined
  );

function simpleAssert(cond, message) {
  if (!cond) {
    throw new Error(message);
  }
}

function enumPropValidator(allow = []) {
  return function validator(val) {
    return allow.includes(val);
  };
}

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  enumPropValidator: enumPropValidator,
  preventObserve: preventObserve,
  simpleAssert: simpleAssert
});

var MglComponentMixin = {
  inject: {
    getMapComponent: {
      default: () => null
    }
  },
  props: {
    // the MglMap component instance
    mapComponent: {
      type: Object
    }
  },
  methods: {
    // the current context
    __context() {
      const component = this.mapComponent || this.getMapComponent();
      const map = component && component.map;
      simpleAssert(component, 'this component requires a MglMap context');
      simpleAssert(map, 'this component requires a MglMap context');
      return {
        map,
        component
      };
    }

  }
};

var MglControlGroup = {
  render(h) {
    // single or no child
    if (this.$slots.default && this.$slots.default.length <= 1) {
      return this.$slots.default;
    } // multi child


    return h('div', {
      class: 'mgl-control-group',
      style: {
        visibility: 'hidden'
      }
    }, this.$slots.default);
  },

  props: {
    position: {
      type: String,
      default: 'bottom-right',
      validator: enumPropValidator(['top-left', 'top-right', 'bottom-left', 'bottom-right'])
    }
  },

  provide() {
    return {
      CONTEXT_POSITION: this.position
    };
  }

};

const BaseControl = {
  mixins: [MglComponentMixin],

  render() {
    return null;
  },

  // inject if default not provided & inject not found
  // Vue will have warn
  inject: {
    CONTEXT_POSITION: {
      default: null
    }
  },
  props: {
    ControlType: {
      type: Function
    },
    getOptions: {
      type: Function
    },
    position: {
      common: true,
      type: String,

      default() {
        return this.CONTEXT_POSITION || 'top-right';
      },

      validator: value => {
        return ['top-left', 'top-right', 'bottom-left', 'bottom-right'].includes(value);
      }
    }
  },

  mounted() {
    const {
      map,
      component
    } = this.__context();

    this.map = map;
    const options = this.getOptions();
    const control = new this.ControlType(options);
    map.addControl(control, this.position); // for delegate methods

    this.control = control;

    for (let event in this.$listeners) {
      control.on(event, (...args) => {
        this.$emit(event, ...args);
      });
    }

    this.$once('hook:beforeDestroy', () => {
      map.removeControl(control);

      for (let event in this.$listeners) {
        control.off(event);
      }
    });
  }

};

const commonKeys = Object.keys(BaseControl.props).filter(k => BaseControl.props[k].common);
const commonProps = _pick(BaseControl.props, commonKeys);
const commonPropsValue = $props => _pick($props, commonKeys);
function createBuiltInControlMixin(ControlType) {
  return {
    inject: BaseControl.inject,
    props: _objectSpread2({}, commonProps),
    methods: {
      getOptions() {
        return _omit(this.$props, commonKeys);
      }

    },

    render(h) {
      return h(BaseControl, {
        props: _objectSpread2({
          ControlType,
          getOptions: this.getOptions
        }, commonPropsValue(this.$props)),
        on: this.$listeners
      });
    }

  };
}

var MglNavigationControl = {
  name: 'MglNavigationControl',
  mixins: [createBuiltInControlMixin(mgl.NavigationControl)],
  props: {
    showCompass: {
      type: Boolean,
      default: true
    },
    showZoom: {
      type: Boolean,
      default: true
    }
  }
};

var MglGeolocateControl = {
  name: 'MglGeolocateControl',
  mixins: [createBuiltInControlMixin(mgl.GeolocateControl)],
  props: {
    positionOptions: {
      type: Object,
      default: () => ({
        enableHighAccuracy: false,
        timeout: 6000
      })
    },
    fitBoundsOptions: {
      type: Object,
      default: () => ({
        maxZoom: 15
      })
    },
    trackUserLocation: {
      type: Boolean,
      default: false
    },
    showUserLocation: {
      type: Boolean,
      default: true
    }
  }
};

var MglAttributionControl = {
  name: 'MglAttributionControl',
  mixins: [createBuiltInControlMixin(mgl.AttributionControl)],
  props: {
    compact: {
      type: Boolean
    },
    customAttribution: {
      type: [String, Array]
    }
  }
};

var MglScaleControl = {
  name: 'MglScaleControl',
  mixins: [createBuiltInControlMixin(mgl.ScaleControl)],
  props: {
    maxWidth: {
      type: Number
    },
    // Unit of the distance ( 'imperial' ,  'metric' or  'nautical' ).
    unit: {
      type: String
    }
  },
  methods: {
    setUnit(...args) {
      return this.$children[0].control.setUnit(...args);
    }

  }
};

var MglFullscreenControl = {
  name: 'MglFullscreenControl',
  mixins: [createBuiltInControlMixin(mgl.FullscreenControl)],
  props: {
    container: {
      type: HTMLElement
    }
  },
  methods: {
    setUnit(...args) {
      return this.$children[0].control.setUnit(...args);
    }

  }
};

//
var script$1 = {
  mixins: [MglComponentMixin],
  components: {
    BaseControl
  },
  props: {
    position: {
      type: String
    },
    containerProps: {
      type: Object
    }
  },

  data() {
    return {
      ready: false
    };
  },

  computed: {
    baseControlProps() {
      return {
        ControlType: this.ControlType,
        getOptions: () => null,
        position: this.position
      };
    },

    ControlType() {
      const $el = this.$el;
      const $container = this.$refs.container;
      return class MglCustomControlType {
        onAdd(map) {
          return $container;
        }

        onRemove() {
          $el.appendChild($container);
        }

        getDefaultPosition() {
          return this.position || 'top-right';
        }

      };
    }

  },

  mounted() {
    this.ready = true;
  },

  methods: {}
};

/* script */
const __vue_script__$1 = script$1;

/* template */
var __vue_render__$1 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "div",
    { staticClass: "custom-control-wrapper", staticStyle: { display: "none" } },
    [
      _vm.ready
        ? _c(
            "BaseControl",
            _vm._b({}, "BaseControl", _vm.baseControlProps, false)
          )
        : _vm._e(),
      _vm._v(" "),
      _c(
        "div",
        _vm._b({ ref: "container" }, "div", _vm.containerProps, false),
        [_vm._t("default")],
        2
      )
    ],
    1
  )
};
var __vue_staticRenderFns__$1 = [];
__vue_render__$1._withStripped = true;

  /* style */
  const __vue_inject_styles__$1 = undefined;
  /* scoped */
  const __vue_scope_id__$1 = undefined;
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$1 = normalizeComponent(
    { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    false,
    undefined,
    undefined,
    undefined
  );

var script$2 = {
  mixins: [MglComponentMixin],
  components: {
    MglCustomControl: __vue_component__$1
  },
  props: {
    position: {
      type: String
    },
    flyOptions: {
      type: Object,

      default() {
        return {
          maxDuration: 200
        };
      }

    }
  },

  data() {
    return {
      // pitch所对应的额度数
      pitch: {
        '2d': 0,
        '3d': 60
      },
      current: '2D'
    };
  },

  mounted() {
    this.map = this.__context().map;
    this.setCurrent();
  },

  methods: {
    // set initial state
    setCurrent() {
      if (this.map.getPitch() === 0) {
        this.current = '2D';
      } else {
        this.current = '3D';
      }
    },

    onClick() {
      if (this.map.getPitch() === this.pitch['2d']) {
        this.to3d();
      } else {
        this.to2d();
      }
    },

    to3d() {
      this.current = '3D';
      this.map.flyTo(_objectSpread2({}, this.flyOptions, {
        pitch: this.pitch['3d']
      }));
    },

    to2d() {
      this.current = '2D';
      this.map.flyTo(_objectSpread2({}, this.flyOptions, {
        pitch: this.pitch['2d']
      }));
    }

  }
};

/* script */
const __vue_script__$2 = script$2;

/* template */
var __vue_render__$2 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "MglCustomControl",
    {
      attrs: {
        position: _vm.position,
        "container-props": { class: "mapboxgl-ctrl mapboxgl-ctrl-group" }
      }
    },
    [
      _c(
        "button",
        {
          staticClass: "mapboxgl-ctrl-icon mapboxgl-ctrl-pitch",
          attrs: { "aria-label": "Pitch", type: "button" },
          on: { click: _vm.onClick }
        },
        [_vm._v("\n    " + _vm._s(_vm.current) + "\n  ")]
      )
    ]
  )
};
var __vue_staticRenderFns__$2 = [];
__vue_render__$2._withStripped = true;

  /* style */
  const __vue_inject_styles__$2 = undefined;
  /* scoped */
  const __vue_scope_id__$2 = undefined;
  /* module identifier */
  const __vue_module_identifier__$2 = undefined;
  /* functional template */
  const __vue_is_functional_template__$2 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$2 = normalizeComponent(
    { render: __vue_render__$2, staticRenderFns: __vue_staticRenderFns__$2 },
    __vue_inject_styles__$2,
    __vue_script__$2,
    __vue_scope_id__$2,
    __vue_is_functional_template__$2,
    __vue_module_identifier__$2,
    false,
    undefined,
    undefined,
    undefined
  );

/**
 * purpose: 点击按钮恢复到一个自定义的 *初始状态*
 */

var script$3 = {
  mixins: [MglComponentMixin],
  components: {
    MglCustomControl: __vue_component__$1
  },
  props: {
    position: {
      type: String
    },
    // use `CameraOptions`
    camera: {
      type: Object
    },
    // use `fitBounds`
    bounds: {
      type: [Array, Object]
    },
    // both `flyTo` & `fitBounds`
    flyOptions: {
      type: Object
    }
  },

  data() {
    return {
      initialCamera: null
    };
  },

  mounted() {
    this.map = this.__context().map;

    if (!this.view) {
      this.initialCamera = {
        zoom: this.map.getZoom(),
        center: this.map.getCenter(),
        bearing: this.map.getBearing(),
        pitch: this.map.getPitch()
      };
    }
  },

  methods: {
    onClick() {
      if (this.camera) {
        return this.map.flyTo(_objectSpread2({}, this.flyOptions, {}, this.camera));
      }

      if (this.bounds) {
        return this.map.fitBounds(this.bounds, this.flyOptions);
      }

      if (this.initialCamera) {
        return this.map.flyTo(_objectSpread2({}, this.flyOptions, {}, this.initialCamera));
      }
    }

  }
};

/* script */
const __vue_script__$3 = script$3;

/* template */
var __vue_render__$3 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "MglCustomControl",
    {
      attrs: {
        position: _vm.position,
        "container-props": { class: "mapboxgl-ctrl mapboxgl-ctrl-group" }
      }
    },
    [
      _c(
        "button",
        {
          staticClass:
            "mapboxgl-ctrl-icon mapboxgl-ctrl-reset-view mapboxgl-ctrl-geolocate",
          attrs: { "aria-label": "Reset view", type: "button" },
          on: { click: _vm.onClick }
        },
        [
          _c("span", {
            staticClass: "mapboxgl-ctrl-icon",
            attrs: { "aria-hidden": "true" }
          })
        ]
      )
    ]
  )
};
var __vue_staticRenderFns__$3 = [];
__vue_render__$3._withStripped = true;

  /* style */
  const __vue_inject_styles__$3 = undefined;
  /* scoped */
  const __vue_scope_id__$3 = undefined;
  /* module identifier */
  const __vue_module_identifier__$3 = undefined;
  /* functional template */
  const __vue_is_functional_template__$3 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$3 = normalizeComponent(
    { render: __vue_render__$3, staticRenderFns: __vue_staticRenderFns__$3 },
    __vue_inject_styles__$3,
    __vue_script__$3,
    __vue_scope_id__$3,
    __vue_is_functional_template__$3,
    __vue_module_identifier__$3,
    false,
    undefined,
    undefined,
    undefined
  );

//
/**
 * https://www.mapbox.com/mapbox-gl-js/api#marker
 */

var script$4 = {
  mixins: [MglComponentMixin],
  props: {
    // marker
    coordinates: {
      type: [Array, Object],
      required: true
    },
    // other marker options
    anchor: {
      type: String,
      default: 'center'
    },
    offset: {
      type: [Object, Array],
      default: () => [0, 0]
    },
    color: {
      type: String,
      default: '#3FB1CE'
    },
    draggable: {
      type: Boolean,
      default: false
    },
    // Popup
    popupOptions: {
      type: Object
    }
  },

  data() {
    return {
      ready: false,
      map: undefined,
      marker: undefined
    };
  },

  computed: {
    // 兼容处理
    initial() {
      return !this.ready;
    }

  },
  watch: {
    // marker position
    coordinates(val) {
      if (!this.ready) return;
      if (!val) return;
      this.marker.setLngLat(val);
    },

    // marker offset
    offset(val) {
      if (!this.ready) return;
      if (!val) return;

      if (this.marker.setOffset) {
        this.marker.setOffset(val);
      }
    },

    draggable(val) {
      if (!this.ready) return;

      if (this.marker.setDraggable) {
        this.marker.setDraggable(val);
      }
    }

  },

  beforeDestroy() {
    if (this.marker) {
      this.marker.remove();
    }

    if (this.popup) {
      this.popup.remove();
    }
  },

  mounted() {
    const {
      map
    } = this.__context();

    this.map = map;
    this.ready = false;

    this._addMarker();

    this._addPopup();

    this.ready = true;
    this.$emit('ready');
  },

  methods: {
    _addMarker() {
      // clean previous
      if (this.marker) {
        this.marker.remove();
      }

      const options = {
        anchor: this.anchor,
        offset: this.offset,
        color: this.color,
        draggable: this.draggable
      };
      const el = this.$slots.marker && this.$slots.marker[0] && this.$slots.marker[0].elm;

      if (el) {
        this.marker = new mgl.Marker(el, options);
      } else {
        this.marker = new mgl.Marker(options);
      }

      this.marker.setLngLat(this.coordinates).addTo(this.map);
    },

    _addPopup() {
      if (!this.$slots.popup || !this.$slots.popup.length) return; // clean previous

      if (this.popup) {
        this.popup.remove();
        this.marker.setPopup(null);
      }

      const node = this.$slots.popup[0].elm;
      this.popup = new mgl.Popup(this.popupOptions).setLngLat(this.coordinates).setDOMContent(node); // bind

      this.marker.setPopup(this.popup);
    },

    remove() {
      this.marker.remove();
    },

    togglePopup() {
      this.marker.togglePopup();
    },

    waitReady() {
      return new Promise(resolve => {
        if (this.ready) return resolve();
        this.$once('ready', resolve);
      });
    }

  }
};

/* script */
const __vue_script__$4 = script$4;

/* template */
var __vue_render__$4 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "div",
    { staticStyle: { display: "none" } },
    [
      _vm._t("marker"),
      _vm._v(" "),
      _vm._t("popup"),
      _vm._v(" "),
      _vm._t("default")
    ],
    2
  )
};
var __vue_staticRenderFns__$4 = [];
__vue_render__$4._withStripped = true;

  /* style */
  const __vue_inject_styles__$4 = undefined;
  /* scoped */
  const __vue_scope_id__$4 = undefined;
  /* module identifier */
  const __vue_module_identifier__$4 = undefined;
  /* functional template */
  const __vue_is_functional_template__$4 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$4 = normalizeComponent(
    { render: __vue_render__$4, staticRenderFns: __vue_staticRenderFns__$4 },
    __vue_inject_styles__$4,
    __vue_script__$4,
    __vue_scope_id__$4,
    __vue_is_functional_template__$4,
    __vue_module_identifier__$4,
    false,
    undefined,
    undefined,
    undefined
  );

var script$5 = {
  mixins: [MglComponentMixin],

  data() {
    return {
      ready: false
    };
  },

  props: {
    // 位置
    lnglat: [Array, Object],
    // 展示
    show: {
      type: Boolean,
      default: false
    },
    // 其他选项
    options: Object
  },
  watch: {
    lnglat(val) {
      if (!this.ready) return;
      this.popup.setLngLat(val);
    },

    show() {
      if (!this.ready) return;
      if (this.show) this.showPopup();else this.hidePopup();
    }

  },

  beforeDestroy() {
    if (this.popup) {
      this.popup.remove();
    }
  },

  mounted() {
    const {
      map
    } = this.__context();

    this.map = map;
    this.init();
    this.ready = true;
  },

  methods: {
    init() {
      this.popup = new mgl.Popup(_objectSpread2({
        closeOnClick: false
      }, this.options));
      this.popup.setDOMContent(this.$el);
      this.popup.setLngLat(this.lnglat); // initial

      if (this.show) {
        this.showPopup();
      }
    },

    showPopup() {
      if (this.popup.isOpen()) return; // show

      this.popup.addTo(this.map);
    },

    hidePopup() {
      if (!this.popup.isOpen()) return; // hide

      this.popup.remove();
    }

  }
};

/* script */
const __vue_script__$5 = script$5;

/* template */
var __vue_render__$5 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "div",
    {
      directives: [
        { name: "show", rawName: "v-show", value: _vm.show, expression: "show" }
      ],
      staticClass: "mgl-popup-container"
    },
    [_vm._t("default")],
    2
  )
};
var __vue_staticRenderFns__$5 = [];
__vue_render__$5._withStripped = true;

  /* style */
  const __vue_inject_styles__$5 = undefined;
  /* scoped */
  const __vue_scope_id__$5 = undefined;
  /* module identifier */
  const __vue_module_identifier__$5 = undefined;
  /* functional template */
  const __vue_is_functional_template__$5 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$5 = normalizeComponent(
    { render: __vue_render__$5, staticRenderFns: __vue_staticRenderFns__$5 },
    __vue_inject_styles__$5,
    __vue_script__$5,
    __vue_scope_id__$5,
    __vue_is_functional_template__$5,
    __vue_module_identifier__$5,
    false,
    undefined,
    undefined,
    undefined
  );

const geojsonOnlyProps = {
  // data
  // A URL to a GeoJSON file, or inline GeoJSON.
  data: {
    type: [String, Object]
  },
  buffer: {
    type: Number
  },
  tolerance: {
    type: Number
  },
  cluster: {
    type: Boolean
  },
  clusterRadius: {
    type: Number
  },
  clusterMaxZoom: {
    type: Number
  },
  clusterProperties: {
    type: Object
  },
  lineMetrics: {
    type: Boolean
  },
  generateId: {
    type: Boolean
  }
};

const props = _objectSpread2({
  id: {
    type: String,

    default() {
      const type = this.type || _get(this, '$options.propsData.type') || 'unknown';
      return `mgl-source-${type}-${this._uid}`;
    }

  },
  type: {
    type: String,
    required: true,
    validator: enumPropValidator(['vector', 'raster', 'raster-dem', 'geojson', 'image', 'video'])
  },
  url: {
    type: String
  },
  tiles: {
    type: Array
  },
  // "type": "raster", Defaults to 512
  tileSize: {
    type: Number
  },
  bounds: {
    type: Array
  },
  scheme: {
    type: String,
    validator: enumPropValidator(['tms', 'xyz'])
  },
  minzoom: {
    type: Number
  },
  maxzoom: {
    type: Number
  },
  attribution: {
    type: String
  },
  // type=raster-dem only
  // https://docs.mapbox.com/mapbox-gl-js/style-spec/#sources-raster-dem-encoding
  // Optional enum. One of "terrarium", "mapbox". Defaults to "mapbox".
  encoding: {
    type: String
  }
}, geojsonOnlyProps, {
  /**
   * image & video
   *
   * image: url & coordinates
   * video: urls & coordinates
   */
  coordinates: {
    type: Array
  },
  urls: {
    type: Array
  }
});

const commonPropKeys = ['id', 'type'];

const selectProps = arr => _pick(props, arr);

const propKeysRegistry = {
  'image': [...commonPropKeys, 'url', 'coordinates'],
  'video': [...commonPropKeys, 'urls', 'coordinates'],
  'vector': [...commonPropKeys, 'url', 'tiles', 'bounds', 'minzoom', 'maxzoom', 'tileSize', 'scheme', 'attribution'],
  'raster': [...commonPropKeys, 'url', 'tiles', 'bounds', 'minzoom', 'maxzoom', 'tileSize', 'scheme', 'attribution'],
  'raster-dem': [...commonPropKeys, 'url', 'tiles', 'bounds', 'minzoom', 'maxzoom', 'tileSize', 'attribution', 'encoding'],
  'geojson': [...commonPropKeys, // only + extra
  ...Object.keys(geojsonOnlyProps), 'maxzoom', 'attribution']
};
const propsRegistry = _mapValues(propKeysRegistry, keys => selectProps(keys));
/**
 * get prop keys by source.type
 */

const getPropKeys = type => propKeysRegistry[type];
var MglSource = {
  name: 'MglSource',
  mixins: [MglComponentMixin],

  // for composition
  // <MglSource>
  //   <MglLayer />
  // </MglSource>
  render(h) {
    // do not render child when source not added
    if (!this.ready) return null; // single or no child

    if (this.$slots.default && this.$slots.default.length <= 1) {
      return this.$slots.default;
    } // multi child


    return h('div', {
      class: 'mgl-source',
      style: {
        visibility: 'hidden'
      }
    }, this.$slots.default);
  },

  provide() {
    return {
      CONTEXT_SOURCE_ID: this.id
    };
  },

  data() {
    return {
      ready: false
    };
  },

  props,
  watch: {
    // image & video
    coordinates: {
      deep: true,

      handler(val) {
        if (!val || !this.ready) return;
        this.getSource().setCoordinates(val);
      }

    },
    // geojson data
    data: {
      deep: true,

      handler(val) {
        if (!val || !this.ready) return;
        this.getSource().setData(val);
      }

    }
  },

  // parent-beforeMount -> child-beforeMount
  // child-destroyed -> parent-destroyed
  beforeMount() {
    const {
      map,
      component
    } = this.__context();

    this.map = map;
    this.component = component; // first add

    this.add(); // add when map reset style load success

    this.component.$on('style-load', this.add);
  },

  destroyed() {
    this.remove();
    this.component && this.component.$off && this.component.$off('style-load', this.add);
  },

  methods: {
    getSource() {
      if (!this.map) return;
      return this.map.getSource(this.id);
    },

    add() {
      if (!this.map) return;

      const sourceKeys = _without(getPropKeys(this.type), 'id');

      const source = _pick(this.$props, sourceKeys);

      for (let k of Object.keys(source)) {
        if (typeof source[k] === 'undefined') delete source[k];
      }

      this.ready = false;
      this.map.addSource(this.id, source); // let vue destroy child and recreate

      this.$nextTick(() => {
        this.ready = true;
        this.$emit('ready');
      });
    },

    remove() {
      if (!this.map || !this.map.getSource(this.id)) return;
      this.map.removeSource(this.id);
    }

  }
};

/**
 * MglLayer
 *
 * usage Example
 * <MglLayer :sourceId='someid'> // use external source
 */

var MglLayer = {
  mixins: [MglComponentMixin],

  render() {
    return null;
  },

  inject: {
    CONTEXT_SOURCE_ID: {
      default: null
    }
  },
  props: {
    id: {
      type: String,

      default() {
        const type = this.type || _get(this, '$options.propsData.type') || 'unknown';
        return `mgl-layer-${this._uid}`;
      }

    },
    type: {
      type: String,
      validator: enumPropValidator(['fill', 'line', 'symbol', 'circle', 'heatmap', 'fill-extrusion', 'raster', 'hillshade', 'background'])
    },
    metadata: {
      type: Object
    },
    // source related
    sourceId: {
      type: String
    },
    sourceLayer: String,
    // layer
    minzoom: Number,
    maxzoom: Number,
    filter: Array,
    paint: Object,
    layout: Object,
    // layer order
    before: String
  },

  data() {
    return {
      ready: false
    };
  },

  watch: {
    filter(val, oldVal) {
      if (!this.ready) return;
      if (_isEqual(val, oldVal)) return;
      this.map.setFilter(this.id, val);
    },

    paint: {
      deep: true,
      handler: function (val, oldVal) {
        if (!this.ready) return;

        for (let prop in val) {
          if (val[prop] !== oldVal[prop]) {
            this.map.setPaintProperty(this.id, prop, val[prop]);
          }
        }
      }
    },
    layout: {
      deep: true,
      handler: function (val, oldVal) {
        if (!this.ready) return;

        for (let prop in val) {
          if (this.layout[prop] !== oldVal[prop]) {
            this.map.setLayoutProperty(this.id, prop, val[prop]);
          }
        }
      }
    }
  },
  computed: {
    finalSourceId() {
      // props
      if (this.sourceId) return this.sourceId; // inject

      if (this.CONTEXT_SOURCE_ID) return this.CONTEXT_SOURCE_ID;
      throw new Error('sourceId empty for the layer');
    },

    // the layer
    layerEntity() {
      const layer = {
        id: this.id,
        type: this.type,
        source: this.finalSourceId
      };
      if (this['source-layer']) layer['source-layer'] = this['source-layer'];
      if (this.paint) layer.paint = this.paint;
      if (this.layout) layer.layout = this.layout;
      if (this.filter) layer.filter = this.filter;
      if (this.maxzoom) layer.maxzoom = this.maxzoom;
      if (this.minzoom) layer.minzoom = this.minzoom;
      return layer;
    }

  },

  beforeMount() {
    const {
      map,
      component
    } = this.__context();

    this.map = map;

    this.__init();
  },

  destroyed() {
    if (!this.ready) return;

    if (this.map.getLayer(this.id)) {
      this.map.removeLayer(this.id);
    }

    this._removeEvent();
  },

  methods: {
    // _init conflicts with Vue
    __init() {
      this._addLayer();

      this.ready = true;
      this.$emit('ready');
    },

    _addLayer() {
      // clean
      if (this.map.getLayer(this.id)) {
        this.map.removeLayer(this.id);
      } // add


      this.map.addLayer(this.layerEntity, this.before); // event

      this._addEvent();
    },

    _addEvent() {
      this.handlers = this.handlers || {};
      const eventNames = Object.keys(this.$listeners);

      for (let name of eventNames) {
        const fn = (...args) => {
          this.$emit(name, ...args);
        };

        this.handlers[name] = fn;
        this.map.on(name, this.id, fn);
      }
    },

    _removeEvent() {
      if (!this.map) return;
      const eventNames = Object.keys(this.$listeners);

      for (let name of eventNames) {
        const fn = this.handlers && this.handlers[name];
        if (!fn) continue;
        this.map.off(name, this.id, fn);
        delete this.handlers[name];
      }
    }

  }
};

var CompositionLayerMixin = {
  mixins: [MglComponentMixin],

  render(h) {
    return h(MglSource, {
      props: this.sourceProps,
      on: {
        ready: () => {
          this.sourceReady = true;
          this.$emit('source-ready');
        }
      }
    }, [h(MglLayer, {
      props: this.layerProps,
      on: {
        ready: () => {
          this.layerReady = true;
          this.$emit('layer-ready');
        }
      }
    })]);
  },

  data() {
    return {
      sourceReady: false,
      layerReady: false
    };
  }

};

// id, type, urls, coordiantes
// id override
// type = video

const propsFromSource = _omit(propsRegistry.video, ['id', 'type']); // layerProps
// id override
// type=raster
// sourceId as self props


const propsFromLayer = _omit(MglLayer.props, ['id', 'type', 'sourceId']);

const props$1 = _objectSpread2({}, propsFromSource, {
  sourceId: {
    type: String,

    default() {
      return `mgl-video-layer-${this._uid}-source`;
    }

  }
}, propsFromLayer, {
  id: {
    type: String,

    default() {
      return `mgl-video-layer-${this._uid}`;
    }

  }
});

const createDeleteMethod = keys => keys.reduce((obj, key) => {
  obj[key] = function (...args) {
    if (!this.map || !this.sourceReady) return;
    const source = this.map.getSource(this.sourceId);
    if (!source) return;
    return source[key](...args);
  };

  return obj;
}, {});

var MglVideoLayer = {
  mixins: [CompositionLayerMixin],
  props: props$1,
  computed: {
    sourceProps() {
      return _objectSpread2({}, _pick(this.$props, Object.keys(propsFromSource)), {
        id: this.sourceId,
        type: 'video'
      });
    },

    layerProps() {
      return _objectSpread2({}, _pick(this.$props, Object.keys(propsFromLayer)), {
        id: this.id,
        type: 'raster',
        sourceId: this.sourceId // 不传, 使用 inject 也可以

      });
    }

  },

  mounted() {
    this.map = this.__context().map;
  },

  methods: _objectSpread2({}, createDeleteMethod(['play', 'pause', 'seek', 'getVideo']))
};

// id, type, url, coordiantes
// id override
// type = image

const propsFromSource$1 = _omit(propsRegistry.image, ['id', 'type']); // layerProps
// id override
// type=raster
// sourceId as self props


const propsFromLayer$1 = _omit(MglLayer.props, ['id', 'type', 'sourceId']);

const props$2 = _objectSpread2({}, propsFromSource$1, {
  sourceId: {
    type: String,

    default() {
      return `mgl-image-layer-${this._uid}-source`;
    }

  }
}, propsFromLayer$1, {
  id: {
    type: String,

    default() {
      return `mgl-image-layer-${this._uid}`;
    }

  }
});

var MglImageLayer = {
  mixins: [CompositionLayerMixin],
  props: props$2,
  computed: {
    sourceProps() {
      return _objectSpread2({}, _pick(this.$props, Object.keys(propsFromSource$1)), {
        id: this.sourceId,
        type: 'image'
      });
    },

    layerProps() {
      return _objectSpread2({}, _pick(this.$props, Object.keys(propsFromLayer$1)), {
        id: this.id,
        type: 'raster',
        sourceId: this.sourceId // 不传, 使用 inject 也可以

      });
    }

  },
  watch: {
    url(val) {
      if (!this.map || !this.sourceReady) return;
      return this.map.getSource(this.sourceId).updateImage({
        url: val
      });
    }

  },

  mounted() {
    this.map = this.__context().map;
  }

};

const renameZoom = prefix => props => _mapKeys(props, (v, k) => {
  if (!['minzoom', 'maxzoom'].includes(k)) return k;
  return `${prefix}${_upperFirst(k)}`;
});

const restoreZoomValue = prefix => obj => _mapKeys(obj, (v, k) => {
  if (k === `${prefix}Minzoom`) return 'minzoom';
  if (k === `${prefix}Maxzoom`) return 'maxzoom';
  return k;
}); // sourceProps
// source.id will be sourceId
// type = raster


const propsFromSource$2 = _flow( // omit
props => _omit(props, ['id', 'type']), // rename
// minzoom => sourceMinzoom
// maxzoom => sourceMaxzoom
renameZoom('source'))(propsRegistry.raster); // layerProps
// id override
// type=raster
// sourceId as self props


const propsFromLayer$2 = _flow( // omit
props => _omit(props, ['id', 'type', 'sourceId']), // rename
renameZoom('layer'))(MglLayer.props);

const props$3 = _objectSpread2({}, propsFromSource$2, {
  sourceId: {
    type: String,

    default() {
      return `mgl-raster-layer-${this._uid}-source`;
    }

  }
}, propsFromLayer$2, {
  id: {
    type: String,

    default() {
      return `mgl-raster-layer-${this._uid}`;
    }

  }
});

var MglRasterLayer = {
  mixins: [CompositionLayerMixin],
  props: props$3,
  computed: {
    sourceProps() {
      return _objectSpread2({}, _flow( // pick
      obj => _pick(obj, Object.keys(propsFromSource$2)), // restore
      restoreZoomValue('source'))(this.$props), {
        id: this.sourceId,
        type: 'raster'
      });
    },

    layerProps() {
      return _objectSpread2({}, _flow( // pick
      obj => _pick(obj, Object.keys(propsFromLayer$2)), // restore
      restoreZoomValue('layer'))(this.$props), {
        id: this.id,
        type: 'raster',
        sourceId: this.sourceId // 不传, 使用 inject 也可以

      });
    }

  }
};

var script$6 = {
  components: {
    MglSource,
    MglLayer
  },
  props: {
    // data
    sourceId: {
      type: String,

      default() {
        return `${this.layerKey || `mgl-polygon-${this._uid}`}-source`;
      }

    },
    geojson: {
      type: Object
    },
    filter: Array,
    // border
    borderColor: String,
    borderWidth: Number,
    borderPaint: Object,
    borderLayout: Object,
    // fill
    fillColor: String,
    fillOpacity: Number,
    fillPaint: Object,
    fillLayout: Object,
    // mglProps 透传给 mgl-layer 的属性
    mglProps: Object,
    // layer 标识
    layerKey: {
      type: String,

      default() {
        return `mgl-polygon-${this._uid}`;
      }

    }
  },
  computed: {
    hasBorder() {
      return this.borderColor || this.borderWidth || this.borderPaint || this.borderLayout;
    },

    hasFill() {
      return this.fillColor || this.fillOpacity || this.fillPaint || this.fillLayout;
    },

    borderLayer() {
      return _objectSpread2({}, this.mglProps, {
        id: this.layerKey + '-border-layer',
        type: 'line',
        sourceId: this.sourceId,
        filter: this.filter,
        layout: this.borderLayout,
        paint: _objectSpread2({
          'line-color': this.borderColor,
          'line-width': this.borderWidth
        }, this.borderPaint)
      });
    },

    fillLayer() {
      return _objectSpread2({}, this.mglProps, {
        id: this.layerKey + '-fill-layer',
        type: 'fill',
        sourceId: this.sourceId,
        filter: this.filter,
        layout: this.fillLayout,
        paint: _objectSpread2({
          'fill-color': this.fillColor,
          'fill-opacity': this.fillOpacity
        }, this.fillPaint)
      });
    }

  }
};

/* script */
const __vue_script__$6 = script$6;

/* template */
var __vue_render__$6 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "MglSource",
    { attrs: { id: _vm.sourceId, type: "geojson", data: _vm.geojson } },
    [
      _vm.hasBorder
        ? _c(
            "MglLayer",
            _vm._b({ key: "border" }, "MglLayer", _vm.borderLayer, false)
          )
        : _vm._e(),
      _vm._v(" "),
      _vm.hasFill
        ? _c(
            "MglLayer",
            _vm._b({ key: "fill" }, "MglLayer", _vm.fillLayer, false)
          )
        : _vm._e()
    ],
    1
  )
};
var __vue_staticRenderFns__$6 = [];
__vue_render__$6._withStripped = true;

  /* style */
  const __vue_inject_styles__$6 = undefined;
  /* scoped */
  const __vue_scope_id__$6 = undefined;
  /* module identifier */
  const __vue_module_identifier__$6 = undefined;
  /* functional template */
  const __vue_is_functional_template__$6 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$6 = normalizeComponent(
    { render: __vue_render__$6, staticRenderFns: __vue_staticRenderFns__$6 },
    __vue_inject_styles__$6,
    __vue_script__$6,
    __vue_scope_id__$6,
    __vue_is_functional_template__$6,
    __vue_module_identifier__$6,
    false,
    undefined,
    undefined,
    undefined
  );

const globalComponents = {};

const globalify = obj => Object.assign(globalComponents, obj); // map
globalify({
  Cedarmaps: __vue_component__
}); // mixin
globalify({
  MglComponentMixin
}); // control
globalify({
  MglControlGroup,
  MglNavigationControl,
  MglGeolocateControl,
  MglAttributionControl,
  MglScaleControl,
  MglFullscreenControl
}); // custom control
globalify({
  MglCustomControl: __vue_component__$1,
  MglPitchControl: __vue_component__$2,
  MglFlyToControl: __vue_component__$3
}); // ui control
globalify({
  MglMarker: __vue_component__$4,
  MglPopup: __vue_component__$5
}); // source
globalify({
  MglSource,
  MglLayer
}); // simple layer
globalify({
  MglVideoLayer,
  MglImageLayer,
  MglRasterLayer,
  MglPolygon: __vue_component__$6
}); // complex layer
// source + layer combination
//
// raster-source + raster-layer
// vector-source/geojson-source + mgl-layer(type=fill/line/symbol/...

const VueMapboxGlPlugin = {
  install(Vue, options) {
    for (let id in globalComponents) {
      const C = globalComponents[id];
      Vue.component(id, C);
    }
  }

};

export default VueMapboxGlPlugin;
export { __vue_component__ as Cedarmaps, MglAttributionControl, MglComponentMixin, MglControlGroup, __vue_component__$1 as MglCustomControl, __vue_component__$3 as MglFlyToControl, MglFullscreenControl, MglGeolocateControl, MglImageLayer, MglLayer, __vue_component__$4 as MglMarker, MglNavigationControl, __vue_component__$2 as MglPitchControl, __vue_component__$6 as MglPolygon, __vue_component__$5 as MglPopup, MglRasterLayer, MglScaleControl, MglSource, MglVideoLayer, globalComponents, index as util };
